==> bsb_neuron-0.0.0b3-py2.py3-none-any.whl: bsb_neuron/cell.py <==
import itertools
from typing import TYPE_CHECKING

from arborize import ModelDefinition, define_model
from bsb import config
from bsb.config.types import object_
from bsb.simulation.cell import CellModel

if TYPE_CHECKING:
    from bsb.morphologies import MorphologySet


@config.dynamic(
    attr_name="model_strategy", required=False, default="arborize", auto_classmap=True
)
class NeuronCell(CellModel):
    def create_instances(
        self, count, ids, pos, morpho: "MorphologySet", rot, additional
    ):
        def dictzip():
            yield from (
                dict(zip(additional.keys(), values[:-1]))
                for values in itertools.zip_longest(
                    *additional.values(), itertools.repeat(count)
                )
            )

        ids, pos, morpho, rot = (
            iter(ids),
            iter(pos),
            iter(morpho),
            iter(rot),
        )
        additer = dictzip()
        return [
            self._create(next(ids), next(pos), next(morpho), next(rot), next(additer))
            for i in range(count)
        ]

    def _create(self, id, pos, morpho, rot, additional):
        if morpho is None:
            raise RuntimeError(
                f"Cell {id} of {self.name} has no morphology, can't use {self.__class__.__name__} to construct it."
            )
        instance = self.create(id, pos, morpho, rot, additional)
        instance.id = id
        return instance


class ArborizeModelTypeHandler(object_):
    @property
    def __name__(self):
        return "arborized model definition"

    def __call__(self, value):
        if isinstance(value, dict):
            model = define_model(value)
            model._cfg_inv = value
            return model
        else:
            return super().__call__(value)

    def __inv__(self, value):
        inv_value = super().__inv__(value)
        if isinstance(inv_value, ModelDefinition):
            # fixme: not good, should at least be converted back to a compatible dict
            #  definition
            return str(inv_value)


@config.node
class ArborizedModel(NeuronCell, classmap_entry="arborize"):
    model = config.attr(type=ArborizeModelTypeHandler(), required=True)
    _schematics = {}

    def create(self, id, pos, morpho, rot, additional):
        from arborize import bsb_schematic, neuron_build

        self.model.use_defaults = True
        schematic = bsb_schematic(morpho, self.model)
        return neuron_build(schematic)


class Shim:
    pass


@config.node
class ShimModel(NeuronCell, classmap_entry="shim"):
    def create(self, id, pos, morpho, rot, additional):
        return Shim()

==> bsb_neuron-4.0.0rc2-py2.py3-none-any.whl: bsb_neuron/cell.py <==
import itertools
from typing import TYPE_CHECKING

from arborize import ModelDefinition, define_model
from bsb import CellModel, config, types

if TYPE_CHECKING:
    from bsb.morphologies import MorphologySet


@config.dynamic(
    attr_name="model_strategy", required=False, default="arborize", auto_classmap=True
)
class NeuronCell(CellModel):
    def create_instances(self, count, ids, pos, morpho: "MorphologySet", rot, additional):
        def dictzip():
            yield from (
                dict(zip(additional.keys(), values[:-1]))
                for values in itertools.zip_longest(
                    *additional.values(), itertools.repeat(count)
                )
            )

        ids, pos, morpho, rot = (
            iter(ids),
            iter(pos),
            iter(morpho),
            iter(rot),
        )
        additer = dictzip()
        return [
            self._create(next(ids), next(pos), next(morpho), next(rot), next(additer))
            for i in range(count)
        ]

    def _create(self, id, pos, morpho, rot, additional):
        if morpho is None:
            raise RuntimeError(
                f"Cell {id} of {self.name} has no morphology, can't use {self.__class__.__name__} to construct it."
            )
        instance = self.create(id, pos, morpho, rot, additional)
        instance.id = id
        return instance


class ArborizeModelTypeHandler(types.object_):
    @property
    def __name__(self):
        return "arborized model definition"

    def __call__(self, value):
        if isinstance(value, dict):
            model = define_model(value)
            model._cfg_inv = value
            return model
        else:
            return super().__call__(value)

    def __inv__(self, value):
        inv_value = super().__inv__(value)
        if isinstance(inv_value, ModelDefinition):
            # fixme: not good, should at least be converted back to a compatible dict
            #  definition
            return str(inv_value)


@config.node
class ArborizedModel(NeuronCell, classmap_entry="arborize"):
    model = config.attr(type=ArborizeModelTypeHandler(), required=True)
    _schematics = {}

    def create(self, id, pos, morpho, rot, additional):
        from arborize import bsb_schematic, neuron_build

        self.model.use_defaults = True
        schematic = bsb_schematic(morpho, self.model)
        return neuron_build(schematic)


class Shim:
    pass


@config.node
class ShimModel(NeuronCell, classmap_entry="shim"):
    def create(self, id, pos, morpho, rot, additional):
        return Shim()
